@@ -4,7 +4,9 @@ module Network.WebSockets
     ( -- * Incoming connections and handshaking
       PendingConnection
     , pendingRequest
+    , AcceptRequest(..)
     , acceptRequest
+    , acceptRequestWith
     , rejectRequest
 
       -- * Main connection type
@@ -29,6 +31,7 @@ module Network.WebSockets
     , Headers
     , Request (..)
     , RequestHead (..)
+    , getRequestSubprotocols
     , Response (..)
     , ResponseHead (..)
 
@@ -42,18 +45,24 @@ module Network.WebSockets
     , HandshakeException (..)
     , ConnectionException (..)
 
-
       -- * Running a standalone server
     , ServerApp
     , runServer
     , runServerWith
 
+      -- * Utilities for writing your own server
+    , makeListenSocket
+    , makePendingConnection
+
       -- * Running a client
     , ClientApp
     , runClient
     , runClientWith
     , runClientWithSocket
     , runClientWithStream
+
+      -- * Utilities
+    , forkPingThread
     ) where
 
 
